<?php

namespace Drupal\course_content\Course\Object;

use Drupal;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\course\Plugin\course\CourseObject\CourseObjectEntity;
use Drupal\node\Entity\Node;
use Drupal\node\Entity\NodeType;
use Drupal\node\NodeInterface;
use stdClass;
use function course_get_course_object;
use function module_load_include;
use function node_type_get_names;

/**
 * A course object that uses a node as a base.
 */
abstract class CourseObjectNode extends CourseObjectEntity {

  /**
   * Course context handler callback.
   */
  public static function context() {
    $route_match = Drupal::routeMatch();
    if ($route_match->getRouteName() == 'entity.node.canonical') {
      $node = $route_match->getParameter('node');

      $type = NodeType::load($node->bundle());
      if ($type->getThirdPartySetting('course_content', 'use')) {
        if (course_get_course_object('content', $node->id())) {
          return array(
            'object_type' => 'content',
            'instance' => $node->id(),
          );
        }
      }

      // This node might not be in a course, so let's check for related nodes.
      $instances = static::getNodeInstances($node);
      if (!empty($instances)) {
        $node = Node::load($instances[0]);
        // @todo this breaks 'content', need to figure this out
        if (course_get_course_object($node->bundle(), $node->id())) {
          return array(
            'object_type' => $node->bundle(),
            'instance' => $node->id(),
          );
        }
      }
    }
  }

  /**
   * When passed a node, this method should return the "parent" nodes that are
   * contained in a course outline.
   *
   * For example, if the passed node was a question in a quiz, all the quiz node
   * IDs should be returned.
   */
  public static function getNodeInstances($node) {
    return array();
  }

  /**
   * Return a list of valid node types.
   *
   * @return array
   *   An array with node type machine names.
   */
  public static function getNodeTypes() {
    $content_types = [];
    $types = NodeType::loadMultiple();
    foreach ($types as $name => $type) {
      if ($type->getThirdPartySetting('course_content', 'use')) {
        $content_types[] = $name;
      }
    }
    return $content_types;
  }




  /**
   * Create a node and set it as this course object's node.
   *
   * @param stdClass $node
   *   A node to be processed for creation, or none to create a generic node. If
   *   a node is provided, it must have at least a type.
   *
   * {@inheritdoc}
   */
  public function createInstance($node = NULL) {
    if ($this->getOption('node_type')) {
      if (!$node) {
        $node = Node::create(['type' => $this->getOption('node_type')]);
      }

      $node->title = $this->getTitle();
      $node->uid = Drupal::currentUser()->id();
      $node->save();
      $this->setInstanceId($node->id());
    }
  }

  public function defaultConfiguration() {
    $defaults = parent::defaultConfiguration();

    $options = array_intersect_key(node_type_get_names(), array_combine($this->getNodeTypes(), $this->getNodeTypes()));
    $defaults['node_type'] = key($options);

    $defaults['clone_and_reference'] = 0;
    return $defaults;
  }

  public function buildConfigurationForm($form, FormStateInterface $form_state) {
    $form += parent::buildConfigurationForm($form, $form_state);

    $config = $this->getConfiguration();

    $types = array_combine($this->getNodeTypes(), $this->getNodeTypes());
    $options = array_intersect_key(node_type_get_names(), $types);

    // Restrict to only configured node types.
    $form['bundle']['#options'] = $options;

    if (!$this->getNodeTypes()) {
      $form['instance']['#disabled'] = TRUE;
      $form['use_existing_node']['#disabled'] = TRUE;
      $form['node_type']['#disabled'] = TRUE;
      $form['configure']['#type'] = 'markup';
      $configure_url = (new Url('entity.node_type.collection'))->toString();
      $form['configure']['#markup'] = '<strong>' . t('You must first <a href="@configure">configure node types</a> to use with Course.', ['@configure' => $configure_url]) . '</strong>';
    }


    if (Drupal::moduleHandler()->moduleExists('clone') && !$this->getInstanceId()) {
      $form['clone_and_reference'] = array(
        '#title' => t('Clone and reference'),
        '#type' => 'checkbox',
        '#description' => t('This will clone the selected content first.'),
        '#default_value' => $config['clone_and_reference'],
        '#weight' => 4,
        '#states' => array(
          'visible' => array(
            ':input[name="use_existing_node"]' => array('checked' => TRUE),
          ),
        ),
      );
    }

    return $form;
  }

  /**
   * Validate the options form. Check the node type.
   */
  public function validateConfigurationForm(&$form, FormStateInterface $form_state) {
    parent::validateConfigurationForm($form, $form_state);
    $nid = $form_state->getValues()['instance'];

    if (empty($nid) && isset($form_state->getValues()['node_type']) && empty($form_state->getValues()['node_type'])) {
      $form_state->setErrorByName('node_type', t('Please select a node type.'));
    }

    if (empty($form_state->getValues()['node_type']) && ($form_state->getValues()['use_existing_node'] || !$this->isTemporary()) && empty($nid)) {
      $form_state->setErrorByName('instance', t('Please select a node.'));
    }
  }



  /**
   * Clone a node before saving.
   *
   * {@inheritdoc}
   */
  function preSave(EntityStorageInterface $storage, $update = TRUE) {

    if ($this->getOption('clone_and_reference')) {
      module_load_include('inc', 'clone', 'clone.pages');
      $new_nid = clone_node_save($this->getOption('instance'));
      $this->setInstanceId($new_nid);
      $this->setOption('clone_and_reference', 0);
    }

    parent::preSave($storage, $update);
  }

  /**
   * Freeze data to persist over cloning/exporting.
   * @return array
   *   An array of data to be frozen.
   */
  function freeze() {
    if ($this->getInstanceId() != $this->getCourse()->getNode()->nid) {
      // Don't freeze the course, if this course is part of the objects.
      $ice = new stdClass;
      $ice->node = $this->getNode();
      return $ice;
    }
  }

  /**
   * Thaw data frozen from an earlier export/clone.
   *
   * @param array $data
   *   Unfrozen data.
   *
   * @return int
   *   The new instance ID.
   */
  function thaw($ice) {
    $node = $ice->node;
    unset($node->nid);
    unset($node->vid);

    // Let other modules do special fixing up.
    $context = array('method' => 'save-edit');
    Drupal::moduleHandler()->alter('clone_node', $node, $context);

    node_save($node);
    $this->setInstanceId($node->id());
    return $this->getInstanceId();
  }

  /**
   * Get this node object's node.
   *
   * @return NodeInterface
   */
  function getNode() {
    return $this->getInstanceEntity();
  }

  /**
   * Deny access to objects without content.
   *
   * @todo conflicts with core D8
   */
  function Xaccess($op = 'view', $account = NULL) {
    if ($op == 'take' && !$this->getNode()) {
      return FALSE;
    }
    return parent::access($op, $account);
  }


  public function getInstanceEntityType(): string {
    return 'node';
  }

}
