<?php

namespace Drupal\course\Entity;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityChangedInterface;
use Drupal\Core\Entity\EntityPublishedInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\RevisionLogInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Link;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\course\Plugin\CourseOutlinePluginManager;
use Drupal\user\EntityOwnerInterface;

interface CourseInterface extends ContentEntityInterface, EntityChangedInterface, EntityOwnerInterface, RevisionLogInterface, EntityPublishedInterface {

  /**
   * Get the course tracker for this course/user.
   *
   * @deprecated use getEnrollment()
   *
   * @return CourseEnrollment
   */
  public function getTracker(AccountInterface $account);
  /**
   * The Drupal path to take this course.
   *
   * @return Url
   */
  public function getUrl();

  /**
   * Set the active CourseObject in this Course.
   *
   * @param int $id
   *   A numeric course object ID.
   */
  public function setActive($id = NULL);

  /**
   * Get the active CourseObject.
   *
   * @return CourseObject
   */
  public function getActive();

  /**
   * Get the next course object, from the active course object.
   *
   * @return CourseObject
   */
  public function getNext();

  /**
   * Get the previous course object, from the active course object.
   *
   * @return CourseObject
   */
  public function getPrev();

  /**
   * Generate navigation links.
   */
  public function getNavigation(AccountInterface $account);

  /**
   * Get the course objects in this course.
   *
   * @return CourseObject[]
   *   An array of course objects.
   */
  public function getObjects();

  function resetCache();

  /**
   * Check if a user is enrolled and active. To retrieve an enrollment that may
   * be inactive, use getEnrollment().
   *
   * @param AccountInterface $account
   *   Account to check.
   *
   * @return bool
   *   If the user is enrolled and active.
   */
  function isEnrolled(AccountInterface $account);

  /**
   * Enroll a user in this course.
   *
   * @param AccountInterface $account
   *   The user to enroll.
   * @param array $values
   *   Any other entity values as they would be passed to
   *   CourseEnrollment::create().
   *
   * @return CourseEnrollment
   *   A saved CourseEnrollment.
   */
  function enroll(AccountInterface $account, $values = []);

  /**
   * Remove a user from this course.
   *
   * @param AccountInterface $account
   */
  function unEnroll(AccountInterface $account);

  /**
   * Load an enrollment.
   *
   * @todo maybe move to CourseStorage
   *
   * @param AccountInterface $account
   *
   * @return \Drupal\course\Entity\CourseEnrollment
   */
  function getEnrollment(AccountInterface $account);

  function getTitle();

  /**
   * Get all access results.
   *
   * @return AccessResult[]
   */
  public function accessMultiple($operation, AccountInterface $account = NULL);
}
