<?php

namespace Drupal\course\Plugin\course\CourseObject;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\GeneratedLink;
use Drupal\Core\Link;
use Drupal\course\Entity\CourseObject;

/**
 * Base class for course objects that reference other entities.
 */
abstract class CourseObjectEntity extends CourseObject implements CourseObjectEntityInterface {

  /**
   * Return the entity type this object is representing.
   *
   * @return string
   */
  abstract public function getInstanceEntityType() : string;

  function getInstanceEntity() : ?EntityInterface {
    if ($this->getInstanceId()) {
      return \Drupal::entityTypeManager()->getStorage($this->getInstanceEntityType())->load($this->getInstanceId());
    }

    return NULL;
  }

  function buildConfigurationForm($form, FormStateInterface $form_state) {
    $form += parent::buildConfigurationForm($form, $form_state);

    $config = $this->getConfiguration();

    $entity_type =  \Drupal::entityTypeManager()->getDefinition($this->getInstanceEntityType());
    $label = $entity_type->getSingularLabel();

    $bundles = \Drupal::service('entity_type.bundle.info')->getBundleInfo($this->getInstanceEntityType());

    $bundle_options = array_column($bundles, 'label');

    $form['use_existing'] = [
      '#type' => 'checkbox',
      '#title' => t('Use existing @label', ['@label' => $label]),
      '#default_value' => (bool) $this->getOption('use_existing'),
      '#weight' => 1,
      '#access' => !$this->getInstanceId(),
    ];

    $form['bundle'] = [
      '#title' => t('Create entity'),
      '#type' => 'select',
      '#options' => $bundle_options,
      '#description' => t('Selecting a bundle will automatically create this @label and link it to this course object.', ['@label' => $label]),
      '#default_value' => $config['bundle'],
      '#states' => [
        'visible' => [
          ':input[name="use_existing"]' => ['checked' => FALSE],
        ],
      ],
      '#weight' => 2,
      '#access' => !$this->getInstanceId(),
    ];
    if (count($bundle_options) > 1) {
      $form['bundle']['#required'] = TRUE;
    }

    $form['instance'] = [
      '#title' => t('Existing @label', ['@label' => $label]),
      '#type' => 'entity_autocomplete',
      '#target_type' => $this->getInstanceEntityType(),
      '#selection_settings' => [
        // In an override, objects could restrict the bundles used by the
        // built-in reference field. For example to restrict the node types
        // being used as course content.
        //'target_bundles' => $this->getNodeTypes(),
      ],
      '#description' => t('Use existing @label instead of creating a new one.', ['@label' => $label]),
      '#default_value' => !empty($this->getInstanceId()) ? $this->getInstanceEntity() : NULL,
      '#maxlength' => 255,
      '#states' => [
        'visible' => [
          ':input[name="use_existing"]' => ['checked' => TRUE],
        ],
      ],
      '#weight' => 3,
    ];

    if ($entity = $this->getInstanceEntity()) {
      $url = $entity->toUrl('canonical', [
        'attributes' => [
          'target' => '_blank',
          'title' => t('Open in new window'),
        ],
      ]);
      $link = Link::fromTextAndUrl(t('@label [id: @id]', ['@label' => $entity->label(), '@id' => $entity->id()]), $url);
      $form['instance']['#description'] = t('Currently set to @link', ['@link' => $link->toString()]);
    }

    $form['private'] = array(
      '#title' => t('Private'),
      '#description' => $this->hasPrivacySupport() ? t('This content will not be available to users who are not enrolled in this course.') : t('You must enable content_access and acl in order to restrict course content to users who are enrolled in this course.'),
      '#type' => 'checkbox',
      '#default_value' => $config['private'],
      '#disabled' => !($this->hasPrivacySupport()),
      '#weight' => 6,
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  function getViewUrl() {
    if ($this->getInstanceEntity()) {
      return $this->getInstanceEntity()->toUrl();
    }
  }

  /**
   * Assume course can apply access control to entities.
   */
  public function hasPrivacySupport() {
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  function defaultConfiguration() {
    $defaults = parent::defaultConfiguration();
    $defaults['private'] = $this->hasPrivacySupport();
    $defaults['bundle'] = '';
    $defaults['use_entity_label'] = 0;
    $defaults['use_existing'] = 0;
    return $defaults;
  }

  /**
   * On object write, set privacy on this node.
   *
   * @todo D7 we used content_access/ACL.
   *
   * {@inheritdoc}
   */
  function postSave(EntityStorageInterface $storage, $update = TRUE) {
    $privacy_enabled = $this->hasPrivacySupport() && $this->getOption('private');
    $external_node = $this->getInstanceId() > 0;
    if ($privacy_enabled && $external_node) {
      \Drupal::entityTypeManager()->getAccessControlHandler('node')->resetCache();
    }

    parent::postSave($storage, $update);
  }

  /**
   * Show a warning if this object has an instance ID, but the entity does not
   * exist.
   */
  function getWarnings() {
    $warnings = parent::getWarnings();
    if (!$this->getInstanceEntity()) {
      $warnings[] = t('The content associated with this object has been deleted.');
    }
    return $warnings;
  }


  /**
   * {@inheritdoc}
   */
  public function getEditUrl() {
    if ($this->getInstanceEntity()) {
      return $this->getInstanceEntity()->toUrl('edit-form');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getTakeUrl() {
    if ($this->getInstanceEntity()) {
      return $this->getInstanceEntity()->toUrl();
    }
  }

  /**
   * Simple node course object behavior is to just redirect to the entity.
   */
  public function getTakeType() {
    return 'redirect';
  }

  /**
   * Get the object title, or return the instance entity's label if the option
   * is set.
   */
  function getTitle() {
    if ($this->getOption('use_entity_label') && $this->getInstanceEntity()) {
      return $this->getInstanceEntity()->label();
    }
    else {
      return parent::getTitle();
    }
  }

 /**
  * Massage the submitted instance value.
  */
 public function submitConfigurationForm(&$form, FormStateInterface $form_state) {
    if ($form_state->getValue('instance')) {
      $id = $form_state->getValue('instance');

      if ($id) {
        $form_state->setValue('instance', $id);
      }
      else {
        // Unset it, or we'll erase the relationship (since the textfield is
        // actually blank).
        $form_state->unsetValue('instance');
      }
    }

    parent::submitConfigurationForm($form, $form_state);
  }

  /**
   * Assume all entities can be cloned.
   *
   * Overrides may prevent cloning behavior.
   */
  function getCloneAbility() {
    return t('%object will be cloned as an entity. Results may vary.', array('%object' => $this->getTitle()));
  }

  /**
   * Destroy the entity instance.
   */
  public function deleteInstance() {
    if ($this->getInstanceEntity()) {
      $this->getInstanceEntity()->delete();
    }
  }


  /**
   * Make the "Edit instance" link use a dialog.
   *
   * {@inheritdoc}
   */
  function getOptionsSummary() {
    $summary = parent::getOptionsSummary();
    if (isset($summary['instance']) && is_a($summary['instance'], GeneratedLink::class)) {
      $url = $this->getEditUrl();
      $url->setOption('query', \Drupal::service('redirect.destination')->getAsArray());
      $url->setOption('attributes', [
        'class' => 'use-ajax',
        'data-dialog-type' => 'modal',
        'data-dialog-options' => Json::encode(['width' => 800]),
      ]);
      $link = Link::fromTextAndUrl(t('Edit instance'), $url)->toString();
      $summary['instance'] = $link;
    }
    return $summary;
  }

}
