<?php

namespace Drupal\Tests\course\Functional;

use Drupal\course\Entity\Course;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;

/**
 * Test class for dealing with adding and removing elements from the course
 * outline.
 *
 * @group course
 */
class CourseOutlineUiTest extends CourseTestBase {

  /**
   * Create a course object through the interface.
   *
   * @param Course $course
   */
  function createCourseObjectUi(Course $course) {
    // Add a new course content object.
    $edit = array();
    $edit["more[object_type]"] = 'course_test_object';
    $this->submitForm($edit, t('Add object'));

    $this->assertSession()->responseContains(t('Test course object'));
  }

  /**
   * Test creating a course object through the UI.
   */
  function testCourseOutlineCrud() {
    $course = $this->createCourse();
    $this->drupalGet("course/{$course->id()}/outline");
    $this->createCourseObjectUi($course);
    $this->clickLink('Settings');
  }

  /**
   * Test maximum course objects per course.
   */
  function testCourseOutlineMaxOccurences() {
    $course = $this->createCourse();
    $this->drupalGet("course/{$course->id()}/outline");
    $this->createCourseObjectUi($course);
    $this->createCourseObjectUi($course);

    $elements = $this->xpath('//select[@id=:id]//option[@value=:option]', array(':id' => 'edit-more-object-type', ':option' => 'course_test_object'));
    $this->assertTrue((bool) $elements, 'User able to add up to maxOccurances of course objects.');

    $this->createCourseObjectUi($course);

    $elements = $this->xpath('//select[@id=:id]//option[@value=:option]', array(':id' => 'edit-more-object-type', ':option' => 'course_test_object'));
    $this->assertFalse((bool) $elements, 'User was not able to add more than maxOccurances of course objects.');
  }

  /**
   * Test that an object can be marked for deletion from the course outline
   * without validation.
   *
   * Disabled for now, see https://www.drupal.org/project/drupalci/issues/3281684
   */
  function xtestObjectDeletion() {
    $this->testCourseOutlineCrud();
    $this->submitForm([], 'edit-delete-button');
    $this->assertSession()->responseContains('Object will be removed from outline');
    $this->drupalGet("course/1/outline");
    $this->submitForm([], t('Save outline'));
    $this->assertSession()->pageTextNotContains('Object will be removed from outline');
  }

  /**
   * Test that a course can be deleted.
   */
  function testDeleteCourse() {
    $course = $this->createCourse(['title' => 'Course 1']);
    $this->drupalGet("course/{$course->id()}/delete");
    $this->submitForm([], t('Delete'));
    $this->assertSession()->pageTextContains('The course Course 1 has been deleted.');
  }

  /**
   * Test custom fields on course object forms.
   *
   * This fails due to https://www.drupal.org/project/drupalci/issues/3281684
   * But works locally.
   */
  function xtestCourseObjectCustomFields() {
    // Add a field to course result and make it required for starting.
    $field_storage = FieldStorageConfig::create([
      'id' => 'course_object.custom_field',
      'field_name' => 'custom_field',
      'entity_type' => 'course_object',
      'type' => 'string',
      'module' => 'core',
    ]);
    $field_storage->save();
    $instance = FieldConfig::create([
      'field_storage' => $field_storage,
      'bundle' => 'course_test_object',
      'label' => 'Object field A',
      'required' => TRUE,
      'field_name' => 'custom_field',
      'entity_type' => 'course_object',
    ]);
    $instance->save();

    \Drupal::service('entity_display.repository')->getFormDisplay('course_object', 'course_test_object', 'default')
      ->setComponent('custom_field', [
        'type' => 'text_textfield',
      ])
      ->save();


    $this->drupalLogin($this->admin_user);
    $course = $this->createCourse();
    $this->drupalGet("course/{$course->id()}/outline");
    $this->createCourseObjectUi($course);
    $this->clickLink('Settings');
    $this->assertSession()->pageTextContains('Object field A');
    $page = $this->getSession()->getPage();
    $page->fillField('edit-custom-field-0-value', 'Test find me');
    $page->pressButton('Update');
    $page->pressButton('Save outline');

    $this->drupalGet("course/{$course->id()}/outline");
    $this->clickLink('Settings');
    $this->assertEquals('Test find me', $page->findField('edit-custom-field-0-value')->getValue());
  }

}
